<?php

namespace Drupal\input_pattern\Plugin\Field\FieldWidget;

use Drupal\Core\Field\Attribute\FieldWidget;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Defines the 'input_pattern' field widget.
 */
#[FieldWidget(
  id: 'input_pattern',
  label: new TranslatableMarkup('Input attributes pattern'),
  field_types: [
    'integer',
    'decimal',
    'bigint',
    'float',
    'email',
    'string',
    'telephone',
  ],
)]
class InputPatternWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings(): array {
    $setting = [
      'type' => 'textfield',
      'placeholder' => '',
      'separator' => '',
      'maxlength' => '',
      'pattern' => '',
      'class' => '',
      'step' => '',
      'list' => '',
      'min' => '',
      'max' => '',
    ];
    return $setting + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $settings = $mode_settings = $this->getSettings();
    $field_settings = $this->getFieldSettings();
    $field_name = $this->fieldDefinition->getName();
    $field_state = 'name="fields[' . $field_name . '][settings_edit_form][settings][type]"';
    $floor = $settings['floor'] ?? NULL;
    $ceil = $settings['ceil'] ?? NULL;
    $base_step = $settings['base_step'] ?? 'any';
    $min = $settings['min'] ?? $floor;
    $max = $settings['max'] ?? $ceil;
    $step_min = $base_step == 'any' ? '> 0' : $base_step;
    $element['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Type for input'),
      '#options' => [
        'textfield' => $this->t('Text'),
        'number' => $this->t('Number'),
        'range' => $this->t('Range number'),
        'search' => $this->t('Search'),
        'tel' => $this->t('Telephone'),
        'email' => $this->t('Email'),
        'url' => $this->t('Url'),
        'week' => $this->t('Week'),
        'month' => $this->t('Month'),
        'time' => $this->t('Time'),
        'date' => $this->t('Date'),
        'datetime-local' => $this->t('Datetime local'),
      ],
      '#default_value' => $this->getSetting('type'),
    ];
    $element['placeholder'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Placeholder'),
      '#default_value' => $this->getSetting('placeholder'),
      '#description' => implode('<br/>', [
        $this->t('Text that will be shown inside the field until a value is entered. This hint is usually a sample value or a brief description of the expected format.'),
        $this->t('Email @pattern', ['@pattern' => 'mail@company.com']),
        $this->t('Telephone @pattern', ['@pattern' => '01 23 45 67 89']),
      ]),
    ];
    $element['pattern'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Regex pattern for input'),
      '#default_value' => $this->getSetting('pattern'),
      '#description' => implode('<br/>', [
        $this->t('Email @pattern', ['@pattern' => '[^@\s]+@[^@\s]+']),
        $this->t('Telephone @pattern', ['@pattern' => '^(\d{2} ?){4}\d{2}$']),
      ]),
      '#states' => [
        'visible' => [
          ":input[$field_state]" => [
            ['value' => 'textfield'],
            ['value' => 'number'],
            ['value' => 'search'],
            ['value' => 'date'],
            ['value' => 'email'],
            ['value' => 'tel'],
            ['value' => 'url'],
          ],
        ],
      ],
    ];
    $element['separator'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Separator'),
      '#default_value' => $this->getSetting('separator'),
      '#description' => $this->t('Specify a character to automatically separate large numbers.'),
      '#attributes' => [
        'maxlength' => 1,
        'size' => 1,
      ],
      '#states' => [
        'visible' => [
          ":input[$field_state]" => [
            ['value' => 'textfield'],
            ['value' => 'number'],
            ['value' => 'search'],
            ['value' => 'tel'],
          ],
        ],
      ],
    ];
    $element['step'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Step', [], ['context' => 'numeric item']),
      '#step' => $base_step,
      '#min' => $base_step == 'any' ? '0' : $base_step,
      '#max' => $max,
      '#default_value' => $this->getSetting('step'),
      '#field_suffix' => "min {$step_min}, max {$max} <mark><strong>Save it first!</strong></mark>",
      '#description' => implode('<br/>', [
        $this->t('The minimum allowed amount to increment or decrement the field value. Note that setting an integer for this value on a decimal or float field restricts the input on the field to integer values only. While updating this field it is recommended to keep the <em>Default Value</em>, <em>Minimum</em> and <em>Maximum</em> fields blank. Leave blank for default = <em>@default</em>.', [
          '@default' => $field_settings['step'] ?? $base_step ?? '',
        ]),
        $this->t('Type telephone use with separator'),
      ]),
      '#states' => [
        'visible' => [
          ":input[$field_state]" => [
            ['value' => 'number'],
            ['value' => 'range'],
            ['value' => 'tel'],
            ['value' => 'week'],
            ['value' => 'month'],
            ['value' => 'time'],
            ['value' => 'date'],
            ['value' => 'datetime-local'],
          ],
        ],
      ],
    ];
    $element['min'] = [
      '#type' => 'number',
      '#title' => $this->t('Minimum', [], ['context' => 'numeric item']),
      '#step' => $settings['step'],
      '#min' => $floor,
      '#max' => $max,
      '#default_value' => $this->getSetting('min'),
      '#field_suffix' => "min {$floor}, max {$max}",
      '#description' => $this->t('The minimum value that should be allowed in this field. Leave blank for default = <em>@default</em>.', [
        '@default' => $field_settings['min'] ?? $floor ?? '',
      ]),
      '#states' => [
        'visible' => [
          ":input[$field_state]" => [
            ['value' => 'number'],
            ['value' => 'range'],
            ['value' => 'week'],
            ['value' => 'month'],
            ['value' => 'time'],
            ['value' => 'date'],
            ['value' => 'datetime-local'],
          ],
        ],
      ],
    ];
    $element['max'] = [
      '#type' => 'number',
      '#title' => $this->t('Maximum', [], ['context' => 'numeric item']),
      '#step' => $settings['step'],
      '#min' => $min,
      '#max' => $ceil,
      '#default_value' => $this->getSetting('max'),
      '#field_suffix' => "min {$min}, max {$ceil}",
      '#description' => $this->t('The maximum value that should be allowed in this field. Leave blank for default = <em>@default</em>.', [
        '@default' => $field_settings['max'] ?? $ceil ?? '',
      ]),
      '#states' => [
        'visible' => [
          ":input[$field_state]" => [
            ['value' => 'number'],
            ['value' => 'range'],
            ['value' => 'week'],
            ['value' => 'month'],
            ['value' => 'time'],
            ['value' => 'date'],
            ['value' => 'datetime-local'],
          ],
        ],
      ],
    ];
    $element['maxlength'] = [
      '#type' => 'number',
      '#title' => $this->t('Max length'),
      '#step' => 1,
      '#min' => $min,
      '#max' => $ceil,
      '#default_value' => $this->getSetting('maxlength'),
      '#states' => [
        'visible' => [
          ":input[$field_state]" => [
            ['value' => 'number'],
            ['value' => 'range'],
            ['value' => 'week'],
            ['value' => 'month'],
            ['value' => 'time'],
            ['value' => 'date'],
            ['value' => 'datetime-local'],
          ],
        ],
      ],
    ];
    $element['size'] = [
      '#type' => 'number',
      '#title' => $this->t('Size'),
      '#step' => 1,
      '#default_value' => $this->getSetting('size'),
      '#states' => [
        'visible' => [
          ":input[$field_state]" => [
            ['value' => 'tel'],
            ['value' => 'url'],
            ['value' => 'email'],
            ['value' => 'search'],
            ['value' => 'password'],
            ['value' => 'textfield'],
          ],
        ],
      ],
    ];
    $element['list'] = [
      '#type' => 'textfield',
      '#title' => $this->t('list'),
      '#step' => 1,
      '#default_value' => $this->getSetting('list'),
    ];
    $element['class'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom class'),
      '#default_value' => $this->getSetting('class'),
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(): array {
    $summary = [];
    $settings = $this->getSettings();
    foreach ($settings as $name => $value) {
      if (!empty($value)) {
        $summary[] = ucfirst($name) . ': ' . $value;
      }
    }
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $settings = $this->getSettings();
    $inputPost = $form_state->getUserInput();
    $field_name = $this->fieldDefinition->getName();
    $skipValidate = FALSE;
    $default_value = $items[$delta]->value ?? NULL;
    if ($settings['type'] == 'number' && !empty($settings['separator']) && !empty($inputPost[$field_name])) {
      $field_value = $inputPost[$field_name];
      $field_value[$delta]['value'] = str_replace($settings['separator'], '', $field_value[$delta]['value']);
      $form_state->setValue($field_name, $field_value);
      $skipValidate = TRUE;
    }
    if ($settings['type'] == 'number' && empty($settings['maxlength']) && !empty($settings['step']) && is_numeric($settings['step'])) {
      $field_settings = $this->getFieldSettings();
      $field_type = $this->fieldDefinition->getFieldStorageDefinition()->getType();
      $limit = strlen($this->getLimitNumber($field_settings, $field_type));
      $settings['maxlength'] = (int) ceil($limit / $settings['step'] + $limit);
    }
    if ($default_value && !empty($settings['separator']) && !empty($settings['step'])) {
      $default_value = preg_replace('/\B(?=(\d{' . $settings['step'] . '})+(?!\d))/', $settings['separator'], $default_value);
      $settings['type'] = 'textfield';
    }
    $element['value'] = $element + [
      '#type' => $settings['type'],
      '#default_value' => $default_value,
      '#attributes' => [
        'class' => ['input-pattern'],
      ],
      '#attached' => [
        'library' => [
          'input_pattern/input_pattern',
        ],
      ],
    ];
    if (!empty($settings['class'])) {
      $element['value']['#attributes']['class'][] = $settings['class'];
      unset($settings['class']);
    }
    foreach (['size', 'step', 'maxlength', 'placeholder'] as $attribute) {
      if (!empty($settings[$attribute])) {
        $element['value']['#' . $attribute] = $settings[$attribute];
      }
    }
    foreach ($settings as $name => $value) {
      if (!empty($value)) {
        $element['value']['#attributes'][$name] = $value;
      }
    }
    if ($skipValidate) {
      $element['value']['#element_validate'] = [];
    }
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    $separator = $this->getSetting('separator');
    $field_settings = $this->getFieldSettings();
    $field_type = $this->fieldDefinition->getFieldStorageDefinition()->getType();
    if (!empty($separator)) {
      foreach ($values as &$value) {
        if (!empty($value['value'])) {
          $value['value'] = str_replace($separator, '', $value['value']);
          if (in_array($field_type, ['decimal', 'float'])) {
            $value['value'] = str_replace(',', '.', $value['value']);
          }
          $limit = $this->getLimitNumber($field_settings, $field_type);
          if (in_array($field_type, [
            'integer',
            'decimal',
            'bigint',
            'float',
          ]) && is_numeric($value['value'])) {
            if ($limit && $value['value'] > $limit) {
              $message = implode(' ', [
                $this->fieldDefinition->getName(),
                $this->t('Numeric value out of range'),
                $limit,
              ]);
              $this->messenger()->addError($message);
              $value['value'] = $limit;
            }
          }
        }
      }
    }
    return parent::massageFormValues($values, $form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getLimitNumber($field_settings, $field_type) {
    $limit = FALSE;
    if (!empty($field_settings["size"])) {
      $limit = match ($field_settings["size"]) {
        'normal' => 2147483647,
        'big' => 9223372036854775807,
      };
    }
    if (in_array($field_type, [
      'decimal',
      'float',
    ]) && !empty($field_settings['precision']) && !empty($field_settings['scale'])) {
      $limit = str_repeat('9', $field_settings['precision'] - $field_settings['scale']) . '.' . str_repeat('9', $field_settings['scale']);
    }
    return $limit;
  }

}
